#!/bin/bash
#
# This script will remove all files whose size exceeds the limit
# specified by the "MAXSIZE" variable (default: 4 Mb)
# Warning: there will be no warning. Files will simply be erased. Use wisely.
#

ONEBYTE=1
ONEKB=$((ONEBYTE*1024))
ONEMB=$((ONEKB*1024))
ONEGB=$((ONEMB*1024))

MAXSIZE=$((1*ONEMB))

SAVED=0
NFILES=0

pushd `dirname $0` > /dev/null
SCRIPTPATH="$( cd "$(dirname "$0")" ; pwd -P )"
popd > /dev/null

for FILE in "$SCRIPTPATH"/*
do
FILESIZE=$(stat -c%s "$FILE")
#echo "Size of $FILE = $FILESIZE bytes."
if [ "$FILESIZE" -gt "$MAXSIZE" ]; then
    rm -f "$FILE"
    NFILES=$((NFILES+1))
    SAVED=$((SAVED+FILESIZE))
fi
done

if [ "$SAVED" -gt "$ONEGB" ]; then
    DSAVED=$((SAVED/1024/1024/1024))
    UNIT="Gb"
elif [ "$SAVED" -gt "$ONEMB" ]; then
    DSAVED=$((SAVED/1024/1024))
    UNIT="Mb"
elif [ "$SAVED" -gt "$ONEKB" ]; then
    DSAVED=$((SAVED/1024))
    UNIT="Mb"
else
    DSAVED=$((SAVED))
    UNIT="Bytes"
fi

echo "Number of files erased:" $NFILES
echo "Saved space:" $DSAVED $UNIT

exit 0
