c     Copyright (C)
c     2014-2018 Centre National de la Recherche Scientifique
c     2014-2018 Mesotar
c     
c     This file must be used under the terms of the CeCILL license.
c     This source file is licensed as described in the file COPYING, which
c     you should have received as part of this distribution.  The terms
c     are also available at
c     http://www.cecill.info/licences/Licence_CeCILL_V2.1-en.txt
c
      subroutine H2O_continuum(Nk_h2o_cont,nu_h2o_cont,k_h2o_cont,
     &     nu,z1,z2,tau)
      implicit none
      include 'max.inc'
c
c     Purpose: to compute the optical thickness due to H2O continuum
c     that results from the 25 cm^-1 truncation of H2O line wings,
c     and computed by AER routine DRCNTNM
c
c     Input:
c       + Nk_h2o_cont: size of the wavenumber grid
c       + nu_h2o_cont: wavenumber grid (inv. cm)
c       + k_h2o_cont: H2O continuum optical thickness over the nu_h2o_cont wavenumber grid
c       + nu: wavenumber (inv. cm) where the optical thickness should be computed
c       + z1: lower altitude of the layer (m)
c       + z2: upper altitude of the layer (m)
c
c     Output:
c       + tau: H2O continuuum optical thickness for nu, between z1 and z2
c
c     I/O
      integer Nk_h2o_cont
      double precision nu_h2o_cont(1:Nk_mx)
      double precision k_h2o_cont(1:Nk_mx)
      double precision nu
      double precision z1,z2
      double precision tau
c     temp
      logical int_found
      integer i,int
      double precision k
c     label
      integer strlen
      character*(Nchar_mx) label
      label='subroutine H2O_continuum'

      int_found=.false.
      do i=1,Nk_h2o_cont-1
         if ((nu_h2o_cont(i).le.nu).and.(nu_h2o_cont(i+1).ge.nu)) then
            int_found=.true.
            int=i
            goto 111
         endif
      enddo ! i
 111  continue
c     Additional error sources
      if (.not.int_found) then
         if (nu.lt.nu_h2o_cont(1)) then
            int_found=.true.
            int=1
         endif
      endif
      if (.not.int_found) then
         if (nu.gt.nu_h2o_cont(Nk_h2o_cont)) then
            int_found=.true.
            int=Nk_h2o_cont-1
         endif
      endif
c
      if (.not.int_found) then
         call error(label)
         write(*,*) 'Wavenumber interval could not be found'
         write(*,*) 'nu_h2o_cont(1)=',
     &        nu_h2o_cont(1)
         write(*,*) 'nu_h2o_cont(',Nk_h2o_cont,')=',
     &        nu_h2o_cont(Nk_h2o_cont)
         write(*,*) 'while nu=',nu
         stop
      else
c     linear interpolation of k_h2o_cont in the identified wavenumber interval
         if ((int.lt.1).and.(Nk_h2o_cont.gt.0)) then
            int=Nk_h2o_cont
         endif
         k=k_h2o_cont(int)
     &        +(k_h2o_cont(int+1)-k_h2o_cont(int))
     &        /(nu_h2o_cont(int+1)-nu_h2o_cont(int))
     &        *(nu-nu_h2o_cont(int))
         if (k.lt.0.0D+0) then
            k=0.0D+0
         endif
c     optical thickness
         tau=k*(z2-z1)
      endif

      return
      end
