c     Copyright (C)
c     2014-2018 Centre National de la Recherche Scientifique
c     2014-2018 Mesotar
c     
c     This file must be used under the terms of the CeCILL license.
c     This source file is licensed as described in the file COPYING, which
c     you should have received as part of this distribution.  The terms
c     are also available at
c     http://www.cecill.info/licences/Licence_CeCILL_V2.1-en.txt
c
      program analytic_RT
      implicit none
      include 'mpif.h'
      include 'max.inc'
      include 'formats.inc'
      include 'param.inc'
      include 'constants.inc'
c
c     Purpose: high-resolution analytic radiative transfer using a plane-parallel physical model
c
c     high-resolution spectra
      character*(Nchar_mx) input_file
      character*(Nchar_mx) hs_base
      integer nis
      integer Hnmol
      integer Hmol_index(1:Nmol_mx)
      integer Hmol_niso(1:Nmol_mx)
      integer Hiso_index(1:Nmol_mx,1:Niso_mx)
      character*10 Hmol_name(1:Nmol_mx)
      double precision Hiso_abundance(1:Nmol_mx,1:Niso_mx)
      double precision Hiso_Qref(1:Nmol_mx,1:Niso_mx)
      integer Hiso_g(1:Nmol_mx,1:Niso_mx)
      double precision Hiso_mass(1:Nmol_mx,1:Niso_mx)
      character*(Nchar_mx) planet_descriptor
      integer Nlev
      double precision alt(1:Nlev_mx)
      double precision pres(1:Nlev_mx)
      double precision temp(1:Nlev_mx)
      double precision capa(1:Nlay_mx)
      integer Nmol
      integer mol_index(1:Nmol_mx)
      integer mol_niso(1:Nmol_mx)
      integer iso_index(1:Nmol_mx,1:Niso_mx)
      character*10 mol_name(1:Nmol_mx)
      double precision abundances(1:Nlev_mx,1:Nmol_mx,1:Niso_mx)
      double precision iso_Qref(1:Nmol_mx,1:Niso_mx)
      integer iso_g(1:Nmol_mx,1:Niso_mx)
      double precision iso_mass(1:Nmol_mx,1:Niso_mx)
      double precision x(1:Nlev_mx,1:Nmol_mx)
      logical co2_found,h2o_found
      double precision x_co2(1:Nlev_mx)
      double precision x_h2o(1:Nlev_mx)
c     options
      logical ai
      logical use_cloud
      logical compute_rayleigh
      logical compute_co2_cia
      logical compute_h2o_cia
      logical compute_N2N2_cia
      logical compute_O2O2_cia
      logical compute_h2o_aer_cont
      logical add_sca2abs
      logical cner,cflux
      logical record_rayleigh
      logical record_co2_cia
      logical record_h2o_cia
      logical record_N2N2_cia
      logical record_O2O2_cia
      logical record_h2o_aer_cont
      logical record_total_ot
      logical record_asr
      logical test_data
c     spectral data
      integer Nf,pass
      character*(Nchar_mx) files(1:Nfiles_mx)
      double precision nu_min,nu_max
      double precision nu_start,nu_end
      integer Nk(1:Nfiles_mx)
      double precision nu0,last_nu
      double precision spectra(1:Nk_mx,1:2,1:Nfiles_mx)
      integer Nk_ref
      double precision nu_ref(1:Nk_mx)
      integer Nk_rt(1:Nfiles_mx)
      double precision spectra_rt(1:Nk_mx,1:2,1:Nfiles_mx)
      double precision nu_hi_out,nu1,nu2,dnu
      double precision pCO2_z1,pCO2_z2
      double precision N2N2_nu_min(1:Ndt_int_mx)
      double precision N2N2_nu_max(1:Ndt_int_mx)
      integer N2N2_ndt,N2N2_np(1:Ndt_int_mx)
      double precision N2N2_T(1:Ndt_int_mx)
      double precision N2N2_cia(1:Ndt_int_mx,1:Np_mx,1:2)
      double precision O2O2_nu_min(1:Ndt_int_mx)
      double precision O2O2_nu_max(1:Ndt_int_mx)
      integer O2O2_ndt,O2O2_np(1:Ndt_int_mx)
      double precision O2O2_T(1:Ndt_int_mx)
      double precision O2O2_cia(1:Ndt_int_mx,1:Np_mx,1:2)
      double precision tau_rayleigh(1:Nk_mx,1:Nlay_mx)
      double precision tau_co2_cia(1:Nk_mx,1:Nlay_mx)
      double precision tau_h2o_cia(1:Nk_mx,1:Nlay_mx)
      double precision tau_N2N2_cia(1:Nk_mx,1:Nlay_mx)
      double precision tau_O2O2_cia(1:Nk_mx,1:Nlay_mx)
      double precision tau_h2o_cont(1:Nk_mx,1:Nlay_mx)
      double precision tau_total(1:Nk_mx,1:Nlay_mx),taug,tauc
      double precision tau_total_cumulated(1:Nk_mx)
c     H2O continuum
      integer Nk_h2o_cont
      double precision nu_h2O_cont(1:Nk_mx)
      double precision h2o_cont(1:Nk_mx)
      double precision k_h2o_cont(1:Nk_mx)
c     radiative transfer
      integer Nlay
      double precision k(1:Nlev_mx)
      double precision k1l(1:Nlev_mx)
      double precision tau_rayleigh_1k(1:Nlay_mx)
      double precision tau_co2_cia_1k(1:Nlay_mx)
      double precision tau_h2o_cia_1k(1:Nlay_mx)
      double precision tau_N2N2_cia_1k(1:Nlay_mx)
      double precision tau_O2O2_cia_1k(1:Nlay_mx)
      double precision tau_h2o_cont_1k(1:Nlay_mx)
      double precision tau(1:Nlay_mx)
      double precision B(0:Nlay_mx)
      double precision Bg,Bs
      double precision Tg,Ts
      double precision epsilon_ground
      double precision abs_cont
      double precision ks_coeff
      double precision probe_pressure
      double precision mu
      integer probe_layer
      double precision probe_altitude
      integer ref
      double precision NER(0:Nlay_mx+1,0:Nlay_mx+1)
      double precision flux_up(0:Nlay_mx)
      double precision flux_dn(0:Nlay_mx)
      double precision flux_net(0:Nlay_mx)
      double precision pflux_up
      double precision pflux_dn
      double precision pflux_net
      double precision budget(1:Nlay_mx)
      double precision gas_ground_budget(1:Nlay_mx)
      double precision gas_space_budget(1:Nlay_mx)
      double precision gas_gas_budget(1:Nlay_mx)
      double precision total_budget(1:Nlay_mx)
      double precision total_gas_ground_budget(1:Nlay_mx)
      double precision total_gas_space_budget(1:Nlay_mx)
      double precision total_gas_gas_budget(1:Nlay_mx)
      double precision total_NER(0:Nlay_mx+1,0:Nlay_mx+1)
      double precision total_flux_up(0:Nlay_mx)
      double precision total_flux_dn(0:Nlay_mx)
      double precision total_flux_net(0:Nlay_mx)
      double precision total_pflux_up
      double precision total_pflux_dn
      double precision total_pflux_net
      double precision ftoa_signal(1:Nk_mx)
      double precision pfup_signal(1:Nk_mx)
      double precision pfdn_signal(1:Nk_mx)
      double precision pfnet_signal(1:Nk_mx)
      double precision gfup_signal(1:Nk_mx)
      double precision gfdn_signal(1:Nk_mx)
      double precision gfnet_signal(1:Nk_mx)
      double precision NER_is(0:Nlay_mx)
      double precision cf(0:Nlay_mx)
      double precision omega(1:Nlay_mx)
      double precision omega_signal(1:Nk_mx,1:Nlay_mx)
      double precision obs_intensity,int_obs_intensity,Tb
      double precision flux_up_signal(1:Nk_mx,0:Nlay_mx)
      double precision flux_dn_signal(1:Nk_mx,0:Nlay_mx)
      double precision flux_net_signal(1:Nk_mx,0:Nlay_mx)
      double precision cf_signal(1:Nk_mx,0:Nlay_mx)
      logical solution_found
      double precision Tb_ik,Tb_signal(1:Nk_mx)
      double precision budget_signal(1:Nk_mx,1:Nlay_mx)
      double precision ground_space_NER_signal(1:Nk_mx)
      double precision gas_ground_budget_signal(1:Nk_mx,1:Nlay_mx)
      double precision gas_space_budget_signal(1:Nk_mx,1:Nlay_mx)
      double precision gas_gas_budget_signal(1:Nk_mx,1:Nlay_mx)
      integer zero,one
      double precision Ze(1:Nk_mx)
      double precision Be(1:Nk_mx)
      double precision Te(1:Nk_mx)
      double precision Tmoy(1:Nk_mx)
c     cloud data
      character*(Nchar_mx) cloud_base
      integer Nb_cloud
      double precision nu_lo_cloud(1:Nb_mx)
      double precision nu_hi_cloud(1:Nb_mx)
      double precision ka_cloud(1:Nlev_mx,1:Nb_mx)
      double precision ks_cloud(1:Nlev_mx,1:Nb_mx)
      double precision g_cloud(1:Nlev_mx,1:Nb_mx)
c     temp
      logical pure_co2
      double precision lambda
      double precision ka_bar(1:Nlay_mx)
      double precision kg(1:Nk_mx)
      integer band,lev
      double precision rho
      logical is_nan,computed,none_computed
      character*(Nchar_mx) outfile
      character*(Nchar_mx) outfile2
      character*(Nchar_mx) data_descriptor
      double precision T(0:Nlay_mx)
      double precision grav_constant
      double precision z1,z2,T1,T2,wavenumber
      double precision p1,p2,tau_tmp
c     MPICH
      integer ierr,errorcode,Np,pindex
      integer stat(MPI_STATUS_SIZE)
      integer code,codep,proc,ret_code
      integer free(1:Nproc_mx),fpf,nsp
      integer Nchunks,chunk
      logical keep_going
      integer ch_idx(1:Nproc_mx)
      integer chunk_k_idx(1:Nchunk_mx)
      integer chunk_lay_idx(1:Nchunk_mx)
      integer first_idx(1:Nproc_mx)
      integer last_idx(1:Nproc_mx)
c     MPICH
c     temp
      integer ik,i,j,lay,ilev
      logical debug
      character*(Nchar_mx) outkfile,command
c     label
      integer strlen
      character*(Nchar_mx) label
      label='program analytic_RT'

c     MPICH
      call MPI_INIT(ierr)
      if (ierr.NE.MPI_SUCCESS) THEN
         call error(label)
         write(*,*) 'MPI could not be initialized'
         call MPI_ABORT(MPI_COMM_WORLD,errorcode,ierr)
      endif
c     Get 'Np', the number of processes
      call MPI_COMM_SIZE(MPI_COMM_WORLD,Np,ierr)
      if (Np.lt.2) then
         call error(label)
         write(*,*) 'Number of processes is:',Np
         write(*,*) 'and it should be greater than 1'
         stop
      endif
      if (Np.gt.Nproc_mx) then
         call error(label)
         write(*,*) 'Number of processes is:',Np
         write(*,*) 'is greater than Nproc_mx=',Nproc_mx
         stop
      endif
c     Get 'pindex', the index of the current process
      call MPI_COMM_RANK(MPI_COMM_WORLD,pindex,ierr)
c     MPICH

c     MPICH
      if (pindex.eq.0) then
c     MPICH
         pass=0
         call init()
         call read_data(nu_min,nu_max,Tg,Ts,epsilon_ground,abs_cont,
     &        ks_coeff,probe_pressure,mu)
         call read_options(ai,use_cloud,
     &        compute_rayleigh,compute_co2_cia,compute_h2o_cia,
     &        compute_N2N2_cia,compute_O2O2_cia,
     &        compute_h2o_aer_cont,add_sca2abs,
     &        cner,cflux,ref,
     &        record_rayleigh,record_co2_cia,record_h2o_cia,
     &        record_N2N2_cia,record_O2O2_cia,record_h2o_aer_cont,
     &        record_total_ot,record_asr,test_data)
c         call read_composition(hs_base,Nlev,alt,pres,temp) ! pres=atm
         input_file='./data/molparam_hitran2008.txt'
         call read_molparam(input_file,
     &        Hnmol,Hmol_index,Hmol_niso,Hmol_name,
     &        Hiso_index,Hiso_abundance,Hiso_Qref,Hiso_g,Hiso_mass)
         call modify_iso_index(Hnmol,Hmol_niso,Hiso_index)
         hs_base='./data/hires_spectra'
         nis=2
         call read_composition(hs_base,
     &        nis,Hnmol,Hmol_index,Hmol_niso,
     &        Hiso_index,Hmol_name,Hiso_abundance,Hiso_Qref,
     &        Hiso_g,Hiso_mass,
     &        planet_descriptor,Nlev,alt,pres,temp, ! pres=atm
     &        Nmol,mol_index,mol_niso,iso_index,
     &        mol_name,abundances,iso_Qref,iso_g,iso_mass,x)
         call get_Cp(Nlev,alt,pres,capa)
         call detect_co2(Nlev,Nmol,mol_name,x,
     &        co2_found,pure_co2,x_co2)
         call detect_h2o(Nlev,Nmol,mol_name,x,
     &        h2o_found,x_h2o)
         call spectrum_files(hs_base,Nf,files,nu_start,nu_end)
         if (Nf.ne.Nlev) then
c     this is the case when not every spectrum has been computed for the current
c     composition file => just ignore the fact there are missing files, and take
c     the actual number of files as the number of levels.
c            Nlev=Nf
            call error(label)
            write(*,*) 'Number of atmospheric levels=',Nlev
            write(*,*) 'while number of HR spectrum files is:',Nf
            stop
         endif
         call init_results(Nlev,Nlay,total_NER,
     &        total_flux_up,total_flux_dn,total_flux_net,
     &        total_pflux_up,total_pflux_dn,total_pflux_net)
c     record temperature profile
         call Tprofile(Nlay,temp,T)
         call record_Tprofile(Nlay,alt,Tg,T,Ts)
c     adjusting limits of the spectral interval for RT if necessary
         call set_nu_limits(nu_min,nu_max,nu_start,nu_end)
c     identify the layer "probe_pressure" is located in, and the corresponding altitude
         call identify_probe_altitude(Nlev,Nlay,
     &        alt,pres,probe_pressure,
     &        probe_layer,probe_altitude)
c     Debug
c         write(*,*) 'probe_layer=',probe_layer
c         write(*,*) 'probe_altitude=',probe_altitude,' m'
c         stop
c     Debug
c     reading cloud data
         if (use_cloud) then
            cloud_base='./data/cloud_data'
            call read_cloud_data(cloud_base,Nlev,Nb_cloud,
     &           nu_lo_cloud,nu_hi_cloud,ka_cloud,ks_cloud,g_cloud)
         else
            do lev=1,Nlev
               do band=1,Nb_cloud
                  ka_cloud(lev,band)=0.0D+0
                  ks_cloud(lev,band)=0.0D+0
                  g_cloud(lev,band)=0.0D+0
               enddo            ! band
            enddo               ! lev
         endif

         last_nu=nu_min
         nu0=nu_min
c     Debug
         outkfile='./results/k001_interpolated.txt'
         command='rm -rf '
     &        //trim(outkfile)
         call exec(command)
c     Debug
         do while (nu0.lt.nu_max)
            pass=pass+1
c     Debug
            write(*,*) '================================'
            write(*,*) 'Iteration: ',pass
            write(*,*) '================================'
            write(*,*) 'Reading HR spectra...'
c     Debug
c     reading part of spectra from files
            call read_kspectra(Nlev,files,nu0,nu_max,
     &           last_nu,Nk,spectra)
            write(*,*) 'Reference discretization...'
c     computing reference wavenumber discretization
            call reference_discretization(Nlev,Nk,spectra,
     &           Nk_ref,nu_ref)
            write(*,*) 'nu=[',nu_ref(1),'-',nu_ref(Nk_ref),']'
c     interpolating the "Nf" spectra along the reference
c     wavenumber discretization
            write(*,*) 'Interpolation of HR spectra...'
            call interpolate_spectra(Nlev,Nk_ref,nu_ref,Nk,spectra,
     &           Nk_rt,spectra_rt,nu_hi_out)
c     ----------------------------------------------------------------
c     within this section, spectra are known along the whole column (spectra_rt)
c     between "nu_ref(1)" and "nu_ref(Nk_ref)", with "Nk(i)" points for each one
c     of the "Nf" spectra.
c     ----------------------------------------------------------------

c     Send all relevant data to every slave process
            if (compute_N2N2_cia) then
               input_file='./data/N2-N2_2011.cia'
               call read_cia_table(input_file,
     &              N2N2_ndt,N2N2_nu_min,N2N2_nu_max,N2N2_np,
     &              N2N2_T,N2N2_cia)
            endif
            if (compute_O2O2_cia) then
               input_file='./data/O2-O2_2011.cia'
               call read_cia_table(input_file,
     &              O2O2_ndt,O2O2_nu_min,O2O2_nu_max,O2O2_np,
     &              O2O2_T,O2O2_cia)
            endif
c
            code=1
            do proc=1,Np-1
               call MPI_SEND(code,1,MPI_INTEGER
     &              ,proc,1,MPI_COMM_WORLD,ierr)
            enddo
c     Debug
c            write(*,*) 'N2N2_ndt=',N2N2_ndt
c     Debug
            call send_global_data(ai,mu,use_cloud,
     &           add_sca2abs,cner,cflux,
     &           ref,Nlev,Nlay,Nk,
     &           alt,pres,temp,Tg,Ts,epsilon_ground,abs_cont,ks_coeff,
     &           probe_pressure,probe_layer,probe_altitude,
     &           nu_ref,spectra_rt,
     &           pure_co2,x_co2,Nmol,mol_name,x,
     &           N2N2_ndt,N2N2_np,N2N2_nu_min,N2N2_nu_max,
     &           N2N2_T,N2N2_cia,
     &           O2O2_ndt,O2O2_np,O2O2_nu_min,O2O2_nu_max,
     &           O2O2_T,O2O2_cia)


c     ====================================================================================
c     Additionnal opacities:
c
c     ------------------------------------------------------------------------------------
c
            none_computed=.true.
c     + Rayleigh scattering optical thickness
            write(*,*) '---'
            if ((compute_rayleigh).and.(co2_found)) then
               write(*,*) 'Computing Rayleigh scattering'
     &              //' optical thickness'
               Nchunks=0
               do ik=1,Nk_ref
                  do lay=1,Nlay
                     Nchunks=Nchunks+1
                     if (Nchunks.gt.Nchunk_mx) then
                        call error(label)
                        write(*,*) 'Nchunks=',Nchunks
                        write(*,*) 'while Nchunk_mx=',Nchunk_mx
                        stop
                     endif
                     chunk_k_idx(Nchunks)=ik
                     chunk_lay_idx(Nchunks)=lay
                  enddo         ! lay
               enddo            ! ik
c
               do proc=1,Np-1
                  free(proc)=1
               enddo
               keep_going=.true.
               chunk=0
c
               do while (keep_going)
 441              continue
                  call find_free_process(free,Np-1,fpf,proc)
                  if (fpf.eq.1) then ! a free child process found
                     chunk=chunk+1
                     if (chunk.le.Nchunks) then
                        ch_idx(proc)=chunk
                        call nu2lambda(nu_ref(chunk_k_idx(chunk)),
     &                       lambda)
                        code=3  ! compute a rayleigh optical thickness
                        call MPI_SEND(code,1,MPI_INTEGER
     &                       ,proc,1,MPI_COMM_WORLD,ierr)
                        call MPI_SEND(chunk_lay_idx(chunk),1,MPI_INTEGER
     &                       ,proc,1,MPI_COMM_WORLD,ierr)
                        call MPI_SEND(lambda,1,MPI_DOUBLE_PRECISION
     &                       ,proc,1,MPI_COMM_WORLD,ierr)
                        free(proc)=0 ! child process index 'proc' is busy
                     else       ! chunk>nchunks
                        free(proc)=2 ! means child process index 'proc' has been stopped
                     endif
                     goto 441
                  else          ! no free child process found
c     checking number of processes that have been stopped
                     call stopped_processes(free,Np-1,nsp)
                     if (nsp.ge.Np-1) then
c     is all processes have been stopped, then exit
                        keep_going=.false.
                        goto 111
                     else
c     otherwise, waiting for news from slave processes...
c     reception of a slave process index: 'proc'
                        call MPI_RECV(proc,1,MPI_INTEGER
     &                       ,MPI_ANY_SOURCE,MPI_ANY_TAG,MPI_COMM_WORLD,
     &                       stat,ierr)
c     reception of a 'ret_code' from process index 'proc'
                        call MPI_RECV(ret_code,1,MPI_INTEGER
     &                       ,proc,MPI_ANY_TAG,MPI_COMM_WORLD,
     &                       stat,ierr)
c
                        if (ret_code.eq.1) then
                           free(proc)=1 ! child process index 'proc' is free again
c     reception of result
                           call MPI_RECV(tau_tmp,1,MPI_DOUBLE_PRECISION
     &                          ,proc,MPI_ANY_TAG,MPI_COMM_WORLD,
     &                          stat,ierr)
                           chunk=ch_idx(proc)
                           tau_rayleigh(chunk_k_idx(chunk),
     &                          chunk_lay_idx(chunk))=tau_tmp
                        else
                           write(*,*) 'Error from ',
     &                          label(1:strlen(label)),' :'
                           write(*,*) 'ret_code=',ret_code
                           stop
                        endif

                     endif      ! nsp=np-1
                  endif         ! fpf=0 or 1
 111              continue
               enddo            ! while keep_going
c
               computed=.true.
               none_computed=.false.
            else
               write(*,*) 'Rayleigh scattering optical thickness'
     &              //' is set to zero because'
               if (.not.compute_rayleigh) then
                  write(*,*) 'its computation was not required'
               else if (.not.co2_found) then
                  write(*,*) 'no CO2 was detected in the gas mixture'
               endif
               call nullify_tau_array(Nk_ref,Nlay,tau_rayleigh)
               computed=.false.
            endif ! compute_rayleigh and co2_found
            if ((computed).and.(record_rayleigh)) then
               write(*,*) 'Recording Rayleigh scattering'
     &              //' optical thickness'
               outfile='./results/tau_rayleigh.txt'
               outfile2='./results/tau_rayleigh_cumulated.txt'
               data_descriptor='tau Rayleigh'
               call record_opacity(pass,
     &              Nk_ref,Nlay,nu_ref,alt,tau_rayleigh,
     &              outfile,outfile2,data_descriptor,test_data)
            else if (computed) then
               write(*,*) 'Rayleigh scattering optical thickness has'
     &              //' been computed, but recording is not required'
            endif               ! record_rayleigh
c
c     ------------------------------------------------------------------------------------
c
c     + CO2 CIA optical thickness
            write(*,*) '---'
            if ((compute_co2_cia).and.(co2_found)) then
               write(*,*) 'Computing CO2 collision-induced absorbtion'
     &              //' optical thickness'
c
               Nchunks=0
               do ik=1,Nk_ref
                  do lay=1,Nlay
                     Nchunks=Nchunks+1
                     if (Nchunks.gt.Nchunk_mx) then
                        call error(label)
                        write(*,*) 'Nchunks=',Nchunks
                        write(*,*) 'while Nchunk_mx=',Nchunk_mx
                        stop
                     endif
                     chunk_k_idx(Nchunks)=ik
                     chunk_lay_idx(Nchunks)=lay
                  enddo         ! lay
               enddo            ! ik
c
               do proc=1,Np-1
                  free(proc)=1
               enddo
               keep_going=.true.
               chunk=0
c
               do while (keep_going)
 442              continue
                  call find_free_process(free,Np-1,fpf,proc)
                  if (fpf.eq.1) then ! a free child process found
                     chunk=chunk+1
                     if (chunk.le.Nchunks) then
                        ch_idx(proc)=chunk
c     Debug
c     write(*,*) 'chunk=',chunk,' /',Nchunks
c     Debug
                        pCO2_z1=pres(chunk_lay_idx(chunk))
     &                       *x_co2(chunk_lay_idx(chunk))
                        pCO2_z2=pres(chunk_lay_idx(chunk)+1)
     &                       *x_co2(chunk_lay_idx(chunk)+1)
                        code=4  ! compute a CO2 collision-induced absorption optical thickness
                        call MPI_SEND(code,1,MPI_INTEGER
     &                       ,proc,1,MPI_COMM_WORLD,ierr)
                        call MPI_SEND(chunk_lay_idx(chunk),1,MPI_INTEGER
     &                       ,proc,1,MPI_COMM_WORLD,ierr)
                        call MPI_SEND(nu_ref(chunk_k_idx(chunk)),1,
     &                       MPI_DOUBLE_PRECISION
     &                       ,proc,1,MPI_COMM_WORLD,ierr)
                        free(proc)=0 ! child process index 'proc' is busy
                     else       ! chunk>nchunks
                        free(proc)=2 ! means child process index 'proc' has been stopped
                     endif
                     goto 442
                  else          ! no free child process found
c     checking number of processes that have been stopped
                     call stopped_processes(free,Np-1,nsp)
                     if (nsp.ge.Np-1) then
c     is all processes have been stopped, then exit
                        keep_going=.false.
                        goto 112
                     else
c     otherwise, waiting for news from slave processes...
c     reception of a slave process index: 'proc'
                        call MPI_RECV(proc,1,MPI_INTEGER
     &                       ,MPI_ANY_SOURCE,MPI_ANY_TAG,MPI_COMM_WORLD,
     &                       stat,ierr)
c     reception of a 'ret_code' from process index 'proc'
                        call MPI_RECV(ret_code,1,MPI_INTEGER
     &                       ,proc,MPI_ANY_TAG,MPI_COMM_WORLD,
     &                       stat,ierr)
c
                        if (ret_code.eq.1) then
                           free(proc)=1 ! child process index 'proc' is free again
c     reception of result
                           call MPI_RECV(tau_tmp,1,MPI_DOUBLE_PRECISION
     &                          ,proc,MPI_ANY_TAG,MPI_COMM_WORLD,
     &                          stat,ierr)
                           chunk=ch_idx(proc)
                           tau_co2_cia(chunk_k_idx(chunk),
     &                          chunk_lay_idx(chunk))=tau_tmp
                        else
                           write(*,*) 'Error from ',
     &                          label(1:strlen(label)),' :'
                           write(*,*) 'ret_code=',ret_code
                           stop
                        endif

                     endif      ! nsp=np-1
                  endif         ! fpf=0 or 1
 112              continue
               enddo            ! while keep_going
c
               computed=.true.
               none_computed=.false.
            else
               write(*,*) 'CO2 collision-induced absorbtion'
     &              //' optical thickness is set to zero because'
               if (.not.compute_co2_cia) then
                  write(*,*) 'its computation was not required'
               else if (.not.co2_found) then
                  write(*,*) 'no CO2 was detected in the gas mixture'
               endif
               call nullify_tau_array(Nk_ref,Nlay,tau_co2_cia)
               computed=.false.
            endif               ! compute_co2_cia and co2_found
            if ((computed).and.(record_co2_cia)) then
               write(*,*) 'Recording CO2 CIA optical thickness'
               outfile='./results/tau_CO2_cia.txt'
               outfile2='./results/tau_CO2_cia_cumulated.txt'
               data_descriptor='tau CO2 CIA'
               call record_opacity(pass,
     &              Nk_ref,Nlay,nu_ref,alt,tau_co2_cia,
     &              outfile,outfile2,data_descriptor,test_data)
            else if (computed) then
               write(*,*) 'CO2 CIA optical thickness has'
     &              //' been computed, but recording is not required'
            endif               ! record_co2_cia
c
c     ------------------------------------------------------------------------------------
c
c     + H2O CIA optical thickness
            write(*,*) '---'
            if ((compute_h2o_cia).and.(h2o_found)) then
               write(*,*) 'Computing H2O collision-induced absorbtion'
     &              //' optical thickness'
c
               Nchunks=0
               do ik=1,Nk_ref
                  do lay=1,Nlay
                     Nchunks=Nchunks+1
                     if (Nchunks.gt.Nchunk_mx) then
                        call error(label)
                        write(*,*) 'Nchunks=',Nchunks
                        write(*,*) 'while Nchunk_mx=',Nchunk_mx
                        stop
                     endif
                     chunk_k_idx(Nchunks)=ik
                     chunk_lay_idx(Nchunks)=lay
                  enddo         ! lay
               enddo            ! ik
c
               do proc=1,Np-1
                  free(proc)=1
               enddo
               keep_going=.true.
               chunk=0
c
               do while (keep_going)
 443              continue
                  call find_free_process(free,Np-1,fpf,proc)
                  if (fpf.eq.1) then ! a free child process found
                     chunk=chunk+1
                     if (chunk.le.Nchunks) then
                        ch_idx(proc)=chunk
c     Debug
c     write(*,*) 'chunk=',chunk,' /',Nchunks
c     Debug
                        code=5  ! compute a H2O collision-induced absorption optical thickness
                        call MPI_SEND(code,1,MPI_INTEGER
     &                       ,proc,1,MPI_COMM_WORLD,ierr)
                        call MPI_SEND(chunk_lay_idx(chunk),1,MPI_INTEGER
     &                       ,proc,1,MPI_COMM_WORLD,ierr)
                        call MPI_SEND(nu_ref(chunk_k_idx(chunk)),1,
     &                       MPI_DOUBLE_PRECISION
     &                       ,proc,1,MPI_COMM_WORLD,ierr)
                        free(proc)=0 ! child process index 'proc' is busy
                     else       ! chunk>nchunks
                        free(proc)=2 ! means child process index 'proc' has been stopped
                     endif
                     goto 443
                  else          ! no free child process found
c     checking number of processes that have been stopped
                     call stopped_processes(free,Np-1,nsp)
                     if (nsp.ge.Np-1) then
c     is all processes have been stopped, then exit
                        keep_going=.false.
                        goto 113
                     else
c     otherwise, waiting for news from slave processes...
c     reception of a slave process index: 'proc'
                        call MPI_RECV(proc,1,MPI_INTEGER
     &                       ,MPI_ANY_SOURCE,MPI_ANY_TAG,MPI_COMM_WORLD,
     &                       stat,ierr)
c     reception of a 'ret_code' from process index 'proc'
                        call MPI_RECV(ret_code,1,MPI_INTEGER
     &                       ,proc,MPI_ANY_TAG,MPI_COMM_WORLD,
     &                       stat,ierr)
c
                        if (ret_code.eq.1) then
                           free(proc)=1 ! child process index 'proc' is free again
c     reception of result
                           call MPI_RECV(tau_tmp,1,MPI_DOUBLE_PRECISION
     &                          ,proc,MPI_ANY_TAG,MPI_COMM_WORLD,
     &                          stat,ierr)
                           chunk=ch_idx(proc)
                           tau_h2o_cia(chunk_k_idx(chunk),
     &                          chunk_lay_idx(chunk))=tau_tmp
                        else
                           write(*,*) 'Error from ',
     &                          label(1:strlen(label)),' :'
                           write(*,*) 'ret_code=',ret_code
                           stop
                        endif

                     endif      ! nsp=np-1
                  endif         ! fpf=0 or 1
 113              continue
               enddo            ! while keep_going
c
               computed=.true.
               none_computed=.false.
            else
               write(*,*) 'H2O collision-induced absorbtion'
     &              //' optical thickness is set to zero because'
               if (.not.compute_h2o_cia) then
                  write(*,*) 'its computation was not required'
               else if (.not.h2o_found) then
                  write(*,*) 'no H2O was detected in the gas mixture'
               endif
               call nullify_tau_array(Nk_ref,Nlay,tau_h2o_cia)
               computed=.false.
            endif               ! compute_h2o_cia and h2o_found
            if ((computed).and.(record_h2o_cia)) then
               write(*,*) 'Recording H2O CIA optical thickness'
               outfile='./results/tau_H2O_cia.txt'
               outfile2='./results/tau_H2O_cia_cumulated.txt'
               data_descriptor='tau H2O CIA'
               call record_opacity(pass,
     &              Nk_ref,Nlay,nu_ref,alt,tau_h2o_cia,
     &              outfile,outfile2,data_descriptor,test_data)
            else if (computed) then
               write(*,*) 'H2O CIA optical thickness has'
     &              //' been computed, but recording is not required'
            endif               ! record_h2o_cia
c
c     ------------------------------------------------------------------------------------
c
c     + N2-N2 CIA optical thickness
            write(*,*) '---'
            if (compute_N2N2_cia) then
               write(*,*) 'Computing N2-N2 collision-induced'
     &              //' absorbtion optical thickness'
c               input_file='./data/N2-N2_2011.cia'
c               call read_cia_table(input_file,
c     &              N2N2_ndt,N2N2_nu_min,N2N2_nu_max,N2N2_np,
c     &              N2N2_T,N2N2_cia)
c
               Nchunks=0
               do ik=1,Nk_ref
                  do lay=1,Nlay
                     Nchunks=Nchunks+1
                     if (Nchunks.gt.Nchunk_mx) then
                        call error(label)
                        write(*,*) 'Nchunks=',Nchunks
                        write(*,*) 'while Nchunk_mx=',Nchunk_mx
                        stop
                     endif
                     chunk_k_idx(Nchunks)=ik
                     chunk_lay_idx(Nchunks)=lay
                  enddo         ! lay
               enddo            ! ik
c
               do proc=1,Np-1
                  free(proc)=1
               enddo
               keep_going=.true.
               chunk=0
c
               do while (keep_going)
 444              continue
                  call find_free_process(free,Np-1,fpf,proc)
                  if (fpf.eq.1) then ! a free child process found
                     chunk=chunk+1
                     if (chunk.le.Nchunks) then
                        ch_idx(proc)=chunk
c     Debug
c     write(*,*) 'chunk=',chunk,' /',Nchunks
c     Debug
                        code=6  ! compute N2-N2 collision-induced absorption optical thickness
                        call MPI_SEND(code,1,MPI_INTEGER
     &                       ,proc,1,MPI_COMM_WORLD,ierr)
                        call MPI_SEND(chunk_lay_idx(chunk),1,MPI_INTEGER
     &                       ,proc,1,MPI_COMM_WORLD,ierr)
                        call MPI_SEND(nu_ref(chunk_k_idx(chunk)),1,
     &                       MPI_DOUBLE_PRECISION
     &                       ,proc,1,MPI_COMM_WORLD,ierr)
                        free(proc)=0 ! child process index 'proc' is busy
                     else       ! chunk>nchunks
                        free(proc)=2 ! means child process index 'proc' has been stopped
                     endif
                     goto 444
                  else          ! no free child process found
c     checking number of processes that have been stopped
                     call stopped_processes(free,Np-1,nsp)
                     if (nsp.ge.Np-1) then
c     is all processes have been stopped, then exit
                        keep_going=.false.
                        goto 114
                     else
c     otherwise, waiting for news from slave processes...
c     reception of a slave process index: 'proc'
                        call MPI_RECV(proc,1,MPI_INTEGER
     &                       ,MPI_ANY_SOURCE,MPI_ANY_TAG,MPI_COMM_WORLD,
     &                       stat,ierr)
c     reception of a 'ret_code' from process index 'proc'
                        call MPI_RECV(ret_code,1,MPI_INTEGER
     &                       ,proc,MPI_ANY_TAG,MPI_COMM_WORLD,
     &                       stat,ierr)
c
                        if (ret_code.eq.1) then
                           free(proc)=1 ! child process index 'proc' is free again
c     reception of result
                           call MPI_RECV(tau_tmp,1,MPI_DOUBLE_PRECISION
     &                          ,proc,MPI_ANY_TAG,MPI_COMM_WORLD,
     &                          stat,ierr)
                           chunk=ch_idx(proc)
                           tau_N2N2_cia(chunk_k_idx(chunk),
     &                          chunk_lay_idx(chunk))=tau_tmp
                        else
                           write(*,*) 'Error from ',
     &                          label(1:strlen(label)),' :'
                           write(*,*) 'ret_code=',ret_code
                           stop
                        endif

                     endif      ! nsp=np-1
                  endif         ! fpf=0 or 1
 114              continue
               enddo            ! while keep_going
c
               computed=.true.
               none_computed=.false.
            else
               write(*,*) 'N2-N2 collision-induced absorbtion'
     &              //' optical thickness is set to zero because'
               if (.not.compute_N2N2_cia) then
                  write(*,*) 'its computation was not required'
               endif
               call nullify_tau_array(Nk_ref,Nlay,tau_N2N2_cia)
               computed=.false.
            endif               ! compute_N2N2_cia
            if ((computed).and.(record_N2N2_cia)) then
               write(*,*) 'Recording N2-N2 CIA optical thickness'
               outfile='./results/tau_N2N2_cia.txt'
               outfile2='./results/tau_N2N2_cia_cumulated.txt'
               data_descriptor='tau N2-N2 CIA'
               call record_opacity(pass,
     &              Nk_ref,Nlay,nu_ref,alt,tau_N2N2_cia,
     &              outfile,outfile2,data_descriptor,test_data)
            else if (computed) then
               write(*,*) 'N2-N2 CIA optical thickness has'
     &              //' been computed, but recording is not required'
            endif               ! record_N2N2_cia
c
c     ------------------------------------------------------------------------------------
c
c     + O2-O2 CIA optical thickness
            write(*,*) '---'
            if (compute_O2O2_cia) then
               write(*,*) 'Computing O2-O2 collision-induced'
     &              //' absorbtion optical thickness'
c               input_file='./data/O2-O2_2011.cia'
c               call read_cia_table(input_file,
c     &              O2O2_ndt,O2O2_nu_min,O2O2_nu_max,O2O2_np,
c     &              O2O2_T,O2O2_cia)
c
               Nchunks=0
               do ik=1,Nk_ref
                  do lay=1,Nlay
                     Nchunks=Nchunks+1
                     if (Nchunks.gt.Nchunk_mx) then
                        call error(label)
                        write(*,*) 'Nchunks=',Nchunks
                        write(*,*) 'while Nchunk_mx=',Nchunk_mx
                        stop
                     endif
                     chunk_k_idx(Nchunks)=ik
                     chunk_lay_idx(Nchunks)=lay
                  enddo         ! lay
               enddo            ! ik
c
               do proc=1,Np-1
                  free(proc)=1
               enddo
               keep_going=.true.
               chunk=0
c
               do while (keep_going)
 445              continue
                  call find_free_process(free,Np-1,fpf,proc)
                  if (fpf.eq.1) then ! a free child process found
                     chunk=chunk+1
                     if (chunk.le.Nchunks) then
                        ch_idx(proc)=chunk
c     Debug
c     write(*,*) 'chunk=',chunk,' /',Nchunks
c     Debug
                        code=7  ! compute a O2-O2 collision-induced absorption optical thickness
                        call MPI_SEND(code,1,MPI_INTEGER
     &                       ,proc,1,MPI_COMM_WORLD,ierr)
                        call MPI_SEND(chunk_lay_idx(chunk),1,MPI_INTEGER
     &                       ,proc,1,MPI_COMM_WORLD,ierr)
                        call MPI_SEND(nu_ref(chunk_k_idx(chunk)),1,
     &                       MPI_DOUBLE_PRECISION
     &                       ,proc,1,MPI_COMM_WORLD,ierr)
                        free(proc)=0 ! child process index 'proc' is busy
                     else       ! chunk>nchunks
                        free(proc)=2 ! means child process index 'proc' has been stopped
                     endif
                     goto 445
                  else          ! no free child process found
c     checking number of processes that have been stopped
                     call stopped_processes(free,Np-1,nsp)
                     if (nsp.ge.Np-1) then
c     is all processes have been stopped, then exit
                        keep_going=.false.
                        goto 115
                     else
c     otherwise, waiting for news from slave processes...
c     reception of a slave process index: 'proc'
                        call MPI_RECV(proc,1,MPI_INTEGER
     &                       ,MPI_ANY_SOURCE,MPI_ANY_TAG,MPI_COMM_WORLD,
     &                       stat,ierr)
c     reception of a 'ret_code' from process index 'proc'
                        call MPI_RECV(ret_code,1,MPI_INTEGER
     &                       ,proc,MPI_ANY_TAG,MPI_COMM_WORLD,
     &                       stat,ierr)
c
                        if (ret_code.eq.1) then
                           free(proc)=1 ! child process index 'proc' is free again
c     reception of result
                           call MPI_RECV(tau_tmp,1,MPI_DOUBLE_PRECISION
     &                          ,proc,MPI_ANY_TAG,MPI_COMM_WORLD,
     &                          stat,ierr)
                           chunk=ch_idx(proc)
                           tau_O2O2_cia(chunk_k_idx(chunk),
     &                          chunk_lay_idx(chunk))=tau_tmp
                        else
                           write(*,*) 'Error from ',
     &                          label(1:strlen(label)),' :'
                           write(*,*) 'ret_code=',ret_code
                           stop
                        endif

                     endif      ! nsp=np-1
                  endif         ! fpf=0 or 1
 115              continue
               enddo            ! while keep_going
c
               computed=.true.
               none_computed=.false.
            else
               write(*,*) 'O2-O2 collision-induced absorbtion'
     &              //' optical thickness is set to zero because'
               if (.not.compute_O2O2_cia) then
                  write(*,*) 'its computation was not required'
               endif
               call nullify_tau_array(Nk_ref,Nlay,tau_O2O2_cia)
               computed=.false.
            endif               ! compute_O2O2_cia
            if ((computed).and.(record_O2O2_cia)) then
               write(*,*) 'Recording O2-O2 CIA optical thickness'
               outfile='./results/tau_O2O2_cia.txt'
               open(11,file=outfile(1:strlen(outfile)))
               outfile2='./results/tau_O2O2_cia_cumulated.txt'
               data_descriptor='tau O2-O2 CIA'
               call record_opacity(pass,
     &              Nk_ref,Nlay,nu_ref,alt,tau_O2O2_cia,
     &              outfile,outfile2,data_descriptor,test_data)
            else if (computed) then
               write(*,*) 'O2-O2 CIA optical thickness has'
     &              //' been computed, but recording is not required'
            endif               ! record_O2O2_cia
c
c     ------------------------------------------------------------------------------------
c
c     + H2O continuum (due to line-wings truncation) -> AER routine
            write(*,*) '---'
            if ((compute_h2o_aer_cont)
     &           .and.(co2_found)
     &           .and.(h2o_found)) then
               write(*,*) 'Computing H2O AER continuum'
     &              //' optical thickness'
               do lay=1,Nlay
c     generate a table of H2O continuum absorption coefficient over the whole
c     spectral range, with a spectral step "dnu=2.0 cm^-1"
                  call AER_water_continuum(
     &              (pres(lay)+pres(lay+1))/2.0D+0,
     &              (temp(lay)+temp(lay+1))/2.0D+0,
     &              (x_h2o(lay)+x_h2o(lay+1))/2.0D+0,
     &              (x_co2(lay)+x_co2(lay+1))/2.0D+0,
     &              nu_ref(1),nu_ref(Nk_ref),
     &              Nk_h2o_cont,nu_h2o_cont,k_h2o_cont)

                  do ik=1,Nk_ref
                     call H2O_continuum(Nk_h2o_cont,
     &                    nu_h2o_cont,k_h2o_cont,
     &                    nu_ref(ik),alt(lay),alt(lay+1),
     &                    tau_h2o_cont(ik,lay))
                  enddo         ! ik
               enddo            ! lay
c
               computed=.true.
               none_computed=.false.
            else
               write(*,*) 'H2O AER continuum absorbtion'
     &              //' optical thickness is set to zero because'
               if (.not.compute_h2o_aer_cont) then
                  write(*,*) 'its computation was not required'
               else if (.not.co2_found) then
                  write(*,*) 'no CO2 has been found in the gas mixture'
               else if (.not.h2o_found) then
                  write(*,*) 'no H2O has been found in the gas mixture'
               endif
               call nullify_tau_array(Nk_ref,Nlay,tau_h2o_cont)
               computed=.false.
            endif               ! compute_h2o_aer_cont and co2_found and h2o_found
            if ((computed).and.(record_h2o_aer_cont)) then
               write(*,*) 'Recording H2O AER continuum'
     &              //' absorption optical thickness'
               outfile='./results/tau_AER_H2O_cont.txt'
               outfile2='./results/tau_AER_H2O_cumulated.txt'
               data_descriptor='tau H2O AER cont.'
               call record_opacity(pass,
     &              Nk_ref,Nlay,nu_ref,alt,tau_h2o_cont,
     &              outfile,outfile2,data_descriptor,test_data)
            else if (computed) then
               write(*,*) 'H2O AER continuum optical thickness has'
     &              //' been computed, but recording is not required'
            endif               ! record_h2o_aer_cont
c
c     ------------------------------------------------------------------------------------
c
c     Sum all optical thicknesses and write them down
            write(*,*) '---'
            if (record_total_ot) then
               write(*,*) 'Computing total optical thickness'
               do ik=1,Nk_ref
                  do lev=1,Nlev
                     k1l(lev)=spectra_rt(ik,2,lev)
                  enddo         ! lev
                  tau_total_cumulated(ik)=0.0D+0
                  do lay=1,Nlay
c     taug: gas opacity for layer "lay" at wavenumber index "ik"
                     if ((k1l(lay).eq.0.0D+0).and.
     &                    (k1l(lay+1).eq.0.0D+0)) then
                        taug=0.0D+0
                     else if ((k1l(lay).gt.0.0D+0).and.
     &                       (k1l(lay+1).eq.0.0D+0)) then
                        taug=k1l(lay)*(alt(lay+1)-alt(lay))/2.0D+0
                     else if ((k1l(lay).gt.0.0D+0).and.
     &                       (k1l(lay+1).gt.0.0D+0)) then
                        taug=(alt(lay+1)-alt(lay))
     &                       *(k1l(lay)-k1l(lay+1))
     &                       /dlog(k1l(lay)/k1l(lay+1))
                     endif
                     tau_total(ik,lay)=taug
     &                    +tau_CO2_cia(ik,lay)
     &                    +tau_H2O_cia(ik,lay)
     &                    +tau_N2N2_cia(ik,lay)
     &                    +tau_O2O2_cia(ik,lay)
     &                    +tau_h2o_cont(ik,lay)
                     if (add_sca2abs) then ! only if required
                        tau_total(ik,lay)=tau_total(ik,lay)
     &                       +tau_rayleigh(ik,lay)/ks_coeff
                     endif
                     tau_total_cumulated(ik)=tau_total_cumulated(ik)
     &                    +tau_total(ik,lay)
                  enddo         ! lay
               enddo            ! ik
c
               write(*,*) 'Recording total optical thickness'
               outfile='./results/tau_total.txt'
               outfile2='./results/tau_total_cumulated.txt'
               data_descriptor='tau total'
               call record_opacity(pass,
     &              Nk_ref,Nlay,nu_ref,alt,tau_total,
     &              outfile,outfile2,data_descriptor,test_data)
            else
               write(*,*) 'Total optical thickness is not recorded'
     &              //' because it was not required'
            endif ! record_total_ot
c
c     ====================================================================================
c
            call set_Nchunks(Nk_ref,Nchunks)

            do proc=1,Np-1
               free(proc)=1
            enddo
            keep_going=.true.
            chunk=0
c
            write(*,*) '---'
            write(*,*) 'Now computing...'
            code=11
            do proc=1,Np-1
               call MPI_SEND(code,1,MPI_INTEGER
     &              ,proc,1,MPI_COMM_WORLD,ierr)
            enddo
            call send_opacities(tau_rayleigh,
     &           tau_co2_cia,tau_h2o_cia,
     &           tau_N2N2_cia,tau_O2O2_cia,tau_h2o_cont,
     &           Nb_cloud,nu_lo_cloud,nu_hi_cloud,
     &           ka_cloud,ks_cloud,g_cloud)
c
            do while (keep_going)
 449           continue
               call find_free_process(free,Np-1,fpf,proc)
               if (fpf.eq.1) then ! a free child process found
                  chunk=chunk+1
                  if (chunk.le.Nchunks) then
                     ch_idx(proc)=chunk
c     Debug
c                     write(*,*) 'chunk=',chunk,' /',Nchunks
c     Debug
                     code=2     ! means a computation must be held
                     call MPI_SEND(code,1,MPI_INTEGER
     &                    ,proc,1,MPI_COMM_WORLD,ierr)
                     free(proc)=0 ! child process index 'proc' is busy
                  else          ! chunk>nchunks
                     free(proc)=2 ! means child process index 'proc' has been stopped
                  endif
                  goto 449
               else             ! no free child process found
c     checking number of processes that have been stopped
                  call stopped_processes(free,Np-1,nsp)
                  if (nsp.ge.Np-1) then
c     is all processes have been stopped, then exit
                     keep_going=.false.
                     goto 119
                  else
c     otherwise, waiting for news from slave processes...
c     reception of a slave process index: 'proc'
                     call MPI_RECV(proc,1,MPI_INTEGER
     &                    ,MPI_ANY_SOURCE,MPI_ANY_TAG,MPI_COMM_WORLD,
     &                    stat,ierr)
c     reception of a 'ret_code' from process index 'proc'
                     call MPI_RECV(ret_code,1,MPI_INTEGER
     &                    ,proc,MPI_ANY_TAG,MPI_COMM_WORLD,
     &                    stat,ierr)

                     if (ret_code.eq.0) then
c     process index 'proc' requires computational load limits to compute something
                        call MPI_SEND(ch_idx(proc)
     &                       ,1,MPI_INTEGER
     &                       ,proc,1,MPI_COMM_WORLD,ierr)
                     else if (ret_code.eq.1) then
                        free(proc)=1 ! child process index 'proc' is free again
c     reception of partial computation results from process index 'proc'
                        call receive_partial_results(proc,
     &                       NER,flux_up,flux_dn,flux_net,
     &                       pflux_up,pflux_dn,pflux_net)
c     update total results
                        if (ch_idx(proc).eq.1) then
                           nu1=nu_ref(ch_idx(proc))
                           nu2=nu_ref(ch_idx(proc)+1)
                        else if (ch_idx(proc).eq.Nk_ref) then
                           nu1=nu_ref(ch_idx(proc)-1)
                           nu2=nu_ref(ch_idx(proc))
                        else
                           nu1=nu_ref(ch_idx(proc))
                           nu2=nu_ref(ch_idx(proc)+1)
                        endif
                        dnu=nu2-nu1
c     total NER matrix
                        if (cner) then
                           call update_ner(Nlev,NER,dnu,total_NER)
                        endif
c     total flux profiles
                        if (cflux) then
                           call update_flux(Nlev,
     &                          flux_up,flux_dn,flux_net, ! W/m^2/cm^-1 if ai=T, W/m^2/cm^-1/sr if ai=F
     &                          pflux_up,pflux_dn,pflux_net ! W/m^2/cm^-1 if ai=T, W/m^2/cm^-1/sr if ai=F
     &                          ,dnu,
     &                          total_flux_up, ! W/m^2 if ai=T, W/m^2/sr if ai=F
     &                          total_flux_dn, ! W/m^2 if ai=T, W/m^2/sr if ai=F
     &                          total_flux_net, ! W/m^2 if ai=T, W/m^2/sr if ai=F
     &                          total_pflux_up, ! W/m^2 if ai=T, W/m^2/sr if ai=F
     &                          total_pflux_dn, ! W/m^2 if ai=T, W/m^2/sr if ai=F
     &                          total_pflux_net) ! W/m^2 if ai=T, W/m^2/sr if ai=F
                        endif
                        ik=ch_idx(proc)
                        if (ik.gt.Nk_mx) then
                           call error(label)
                           write(*,*) 'ik=',ik
                           write(*,*) 'while Nk_mx=',Nk_mx
                           stop
                        else
c     flux signals
                           if (cflux) then
                              do i=0,Nlev-1
                                 flux_up_signal(ik,i)=flux_up(i) ! W/m^2/cm^-1 if ai=T, W/m^2/cm^-1/sr if ai=F
                                 flux_dn_signal(ik,i)=flux_dn(i) ! W/m^2/cm^-1 if ai=T, W/m^2/cm^-1/sr if ai=F
                                 flux_net_signal(ik,i)=flux_net(i) ! W/m^2/cm^-1 if ai=T, W/m^2/cm^-1/sr if ai=F
                              enddo ! i
                              ftoa_signal(ik)=flux_up(Nlev-1)
     &                             -flux_dn(Nlev-1) ! W/m^2/cm^-1 if ai=T, W/m^2/cm^-1/sr if ai=F
                              pfup_signal(ik)=pflux_up ! W/m^2/cm^-1 if ai=T, W/m^2/cm^-1/sr if ai=F
                              pfdn_signal(ik)=pflux_dn ! W/m^2/cm^-1 if ai=T, W/m^2/cm^-1/sr if ai=F
                              pfnet_signal(ik)=pflux_net ! W/m^2/cm^-1 if ai=T, W/m^2/cm^-1/sr if ai=F
                              gfup_signal(ik)=flux_up_signal(ik,0) ! W/m^2/cm^-1 if ai=T, W/m^2/cm^-1/sr if ai=F
                              gfdn_signal(ik)=flux_dn_signal(ik,0) ! W/m^2/cm^-1 if ai=T, W/m^2/cm^-1/sr if ai=F
                              gfnet_signal(ik)=flux_net_signal(ik,0) ! W/m^2/cm^-1 if ai=T, W/m^2/cm^-1/sr if ai=F
                           else
                              call analytic_net_fluxes(Nlev-1,NER,
     &                             flux_net) ! W/m^2/cm^-1 if ai=T, W/m^2/cm^-1/sr if ai=F
                              do i=0,Nlev-1
                                 flux_net_signal(ik,i)=flux_net(i) ! W/m^2/cm^-1 if ai=T, W/m^2/cm^-1/sr if ai=F
                              enddo ! i
                              ftoa_signal(ik)=flux_net(Nlev-1) ! W/m^2/cm^-1 if ai=T, W/m^2/cm^-1/sr if ai=F
                           endif
c     contribution function
                           call contribution_function(Nlay,NER,alt,
     &                          cf,omega)
                           do i=0,Nlay
                              cf_signal(ik,i)=cf(i)
                           enddo ! i
                           do i=1,Nlay
                              omega_signal(ik,i)=omega(i)
                           enddo ! i
c     output from single-glass model
                           call glass_model(Nlay,alt,temp,Tg,Ts,
     &                          nu_ref(ik),omega,
     &                          Ze(ik),Be(ik),Te(ik),Tmoy(ik))
c     Debug
c                           write(*,*) ik,Be(ik),Te(ik),Tmoy(ik)
c     Debug
c     brightness temperature
                           call net_flux_toa_to_obs_intensity(ai,
     &                          ftoa_signal(ik), ! W/m^2/cm^-1 if ai=T, W/m^2/cm^-1/sr if ai=F
     &                          obs_intensity) ! W/m^2/cm^-1/sr
                           call solve_Tb_wavnb(nu_ref(ik),
     &                          obs_intensity, ! W/m^2/cm^-1/sr
     &                          solution_found,
     &                          Tb_ik) ! K
                           if (solution_found) then
                              Tb_signal(ik)=Tb_ik
                           endif
c     budget signals
                           if (cner) then
                              call analytic_budgets_from_ner(ai,Nlev-1,
     &                             NER,
     &                             budget,
     &                             gas_ground_budget,
     &                             gas_space_budget,
     &                             gas_gas_budget)
                              ground_space_NER_signal(ik)=
     &                             NER(0,Nlay+1)
                              do i=1,Nlev-1
                                 budget_signal(ik,i)=budget(i)
                                 gas_ground_budget_signal(ik,i)=
     &                                gas_ground_budget(i)
                                 gas_space_budget_signal(ik,i)=
     &                                gas_space_budget(i)
                                 gas_gas_budget_signal(ik,i)=
     &                                gas_gas_budget(i)
                              enddo ! i
                           else if (cflux) then
                              call analytic_budgets_from_net_fluxes(
     &                             Nlev-1,flux_net,
     &                             budget)
                              do i=1,Nlev-1
                                 budget_signal(ik,i)=budget(i)
                              enddo ! i
                           endif
                        endif
                     else
                        write(*,*) 'Error from ',
     &                       label(1:strlen(label)),' :'
                        write(*,*) 'ret_code=',ret_code
                        stop
                     endif

                  endif         ! nsp=np-1
               endif            ! fpf=0 or 1
 119           continue
            enddo               ! while keep_going
            write(*,*) 'Done; now recording results...'
c     record cos(theta)
            if (.not.ai) then
               open(11,file='./results/cos_theta')
               write(11,*) mu
               close(11)
            endif
c     append flux signals to their respective output files
            call record_ftoa_signal(pass,ai,Nk_ref,nu_ref,ftoa_signal) ! W/m^2/cm^-1 if ai=T, W/m^2/cm^-1/sr if ai=F
            call record_pflux_signal(pass,ai,Nk,nu_ref,
     &           pfup_signal,pfdn_signal,pfnet_signal) ! W/m^2/cm^-1 if ai=T, W/m^2/cm^-1/sr if ai=F
            call record_gflux_signal(pass,ai,Nk,nu_ref,
     &           gfup_signal,gfdn_signal,gfnet_signal) ! W/m^2/cm^-1 if ai=T, W/m^2/cm^-1/sr if ai=F
            call record_cf_signal(pass,Nlev,Nlay,alt,Nk_ref,nu_ref,
     &           cf_signal,omega_signal,Ze,Be,Te,Tmoy)
            call record_Tb_signal(pass,Nk_ref,nu_ref,Tb_signal)
            if (cflux) then
               call record_flux_up_signal(pass,Nk_ref,nu_ref,
     &              Nlev,alt,pres,
     &              flux_up_signal)
               call record_flux_dn_signal(pass,Nk_ref,nu_ref,
     &              Nlev,alt,pres,
     &              flux_dn_signal)
               call record_flux_net_signal(pass,Nk_ref,nu_ref,
     &              Nlev,alt,pres,
     &              flux_net_signal)
            else
               if (cner) then
                  call record_flux_net_signal(pass,Nk_ref,nu_ref,
     &                 Nlev,alt,pres,
     &                 flux_net_signal)
               endif
            endif
c     append budget signals to their respective output files
            if (cner) then
               if (record_asr) then
                  call record_budget_signal(pass,Nk_ref,nu_ref,
     &                 Nlev,alt,pres,capa,budget_signal)
                  call record_ground_space_NER_signal(pass,
     &                 Nk,nu_ref,
     &                 ground_space_NER_signal,flux_up_signal,
     &                 tau_total_cumulated)
                  call record_gas_ground_budget_signal(pass,
     &                 Nk_ref,nu_ref,
     &                 Nlev,alt,pres,capa,gas_ground_budget_signal)
                  call record_gas_space_budget_signal(pass,
     &                 Nk_ref,nu_ref,
     &                 Nlev,alt,pres,capa,gas_space_budget_signal)
                  call record_gas_gas_budget_signal(pass,
     &                 Nk_ref,nu_ref,
     &                 Nlev,alt,pres,capa,gas_gas_budget_signal)
               endif ! record_asr
               call record_ES_signal(pass,Nk,nu_ref,
     &              ground_space_NER_signal,flux_up_signal,
     &              tau_total_cumulated)
            else if (cflux) then
               if (record_asr) then
                  call record_budget_signal(pass,Nk_ref,nu_ref,
     &                 Nlev,alt,pres,capa,budget_signal)
               endif ! record_asr
            endif

c     ----------------------------------------------------------------
            nu0=nu_ref(Nk_ref)
         enddo                  ! while nu0 < nu_max
c     finalize total results
c         if ((cner).or.(cflux)) then
c            call finalize_results(Nlev,nu_min,nu_max,
c     &           total_NER,total_flux_up,total_flux_dn,total_flux_net)
c         endif
c     record total results
         if (cner) then
            call record_ner(ai,Nlev,nu_min,nu_max,total_NER)
         endif
         if (cflux) then
            call record_flux_up(ai,Nlev,alt,pres,total_flux_up)
            call record_flux_dn(ai,Nlev,alt,pres,total_flux_dn)
            call record_flux_net(ai,Nlev,alt,pres,total_flux_net)
            call record_total_pflux(ai,probe_pressure,probe_altitude,
     &           total_pflux_up,total_pflux_dn,total_pflux_net)
         else
            if (cner) then
               call analytic_net_fluxes(Nlev-1,total_NER,total_flux_net)
               call record_flux_net(ai,Nlev,alt,pres,total_flux_net)
            endif
         endif
c     radiative budgets
         if (cner) then
            call analytic_budgets_from_ner(ai,Nlev-1,total_NER,
     &           total_budget,
     &           total_gas_ground_budget,
     &           total_gas_space_budget,
     &           total_gas_gas_budget)
            write(*,*) 'Recording budget from NERs...'
            call record_budget(Nlev,alt,capa,total_budget)
            call record_gas_ground_budget(Nlev,alt,capa,
     &           total_gas_ground_budget)
            call record_gas_space_budget(Nlev,alt,capa,
     &           total_gas_space_budget)
            call record_gas_gas_budget(Nlev,alt,capa,
     &           total_gas_gas_budget)
         else if (cflux) then
            call analytic_budgets_from_net_fluxes(Nlev-1,total_flux_net,
     &           total_budget)
            write(*,*) 'Recording budget from fluxes...'
            call record_budget(Nlev,alt,capa,total_budget)
         endif
c     stopping all child processes
         do proc=1,np-1
            code=0
            call MPI_SEND(code,1,MPI_INTEGER,
     &           proc,1,MPI_COMM_WORLD,ierr)
         enddo
c     cleaning temporary files
         call clean()

      else ! every other process
         zero=0
         one=1
 333     continue
c     wait for "codep" from root process
         call MPI_RECV(codep,1,MPI_INTEGER
     &        ,0,MPI_ANY_TAG,MPI_COMM_WORLD,stat,ierr)

c     codep=0 means exit
         if (codep.eq.0) then
            goto 666
c     codep=1 means data reception for solar computation
         else if (codep.eq.1) then
            call receive_global_data(ai,mu,use_cloud,
     &           add_sca2abs,cner,cflux,
     &           ref,Nlev,Nlay,Nk,
     &           alt,pres,temp,Tg,Ts,epsilon_ground,abs_cont,ks_coeff,
     &           probe_pressure,probe_layer,probe_altitude,
     &           nu_ref,spectra_rt,
     &           pure_co2,x_co2,Nmol,mol_name,x,
     &           N2N2_ndt,N2N2_np,N2N2_nu_min,N2N2_nu_max,
     &           N2N2_T,N2N2_cia,
     &           O2O2_ndt,O2O2_np,O2O2_nu_min,O2O2_nu_max,
     &           O2O2_T,O2O2_cia)
c     Debug
c            write(*,*) 'proc=',proc,' N2N2_ndt=',N2N2_ndt
c     Debug
         else if (codep.eq.11) then
            call receive_opacities(tau_rayleigh,
     &           tau_co2_cia,tau_h2o_cia,
     &           tau_N2N2_cia,tau_O2O2_cia,tau_h2o_cont,
     &           Nb_cloud,nu_lo_cloud,nu_hi_cloud,
     &           ka_cloud,ks_cloud,g_cloud)
c     codep=2 means the process has to run a computation
         else if (codep.eq.2) then
c     sending process index to process 0
            call MPI_SEND(pindex,1,MPI_INTEGER,0,1,MPI_COMM_WORLD,ierr)
c     sending integer '0' to process 0, meaning computational load limits are required
            call MPI_SEND(zero,1,MPI_INTEGER,0,1,MPI_COMM_WORLD,ierr)
c     receiving sector index from master process
            call MPI_RECV(ik,1,MPI_INTEGER
     &           ,0,MPI_ANY_TAG,MPI_COMM_WORLD,
     &           stat,ierr)
c     COMPUTATION
c     use "ik" to compute Bg, Bs, B, tau, ...
            do i=1,Nlev
               k(i)=spectra_rt(ik,2,i)
            enddo               ! i
            do lay=1,Nlay
               tau_rayleigh_1k(lay)=tau_rayleigh(ik,lay)
               tau_co2_cia_1k(lay)=tau_co2_cia(ik,lay)
               tau_h2o_cia_1k(lay)=tau_h2o_cia(ik,lay)
               tau_N2N2_cia_1k(lay)=tau_N2N2_cia(ik,lay)
               tau_O2O2_cia_1k(lay)=tau_O2O2_cia(ik,lay)
               tau_h2o_cont_1k(lay)=tau_h2o_cont(ik,lay)
            enddo ! i
c     Debug
c     if (nu_ref(ik).eq.4000.0D+0) then
c     debug=.true.
c     else
            debug=.false.
c     endif
c     Debug
            call profiles(debug,use_cloud,add_sca2abs,
     &           Nlev,nu_ref(ik),k,alt,pres,temp,Tg,Ts,
     &           abs_cont,ks_coeff,
     &           tau_rayleigh_1k,tau_co2_cia_1k,tau_h2o_cia_1k,
     &           tau_N2N2_cia_1k,tau_O2O2_cia_1k,tau_h2o_cont_1k,
     &           Nb_cloud,nu_lo_cloud,nu_hi_cloud,
     &           ka_cloud,ks_cloud,g_cloud,
     &           Nlay,Bg,Bs,B,tau)
c     Debug
c            write(*,*) ik,nu_ref(ik)
c     &           ,tau(1)
c     &           ,tau_N2N2_cia_1k(1)
c     &           ,tau_O2O2_cia_1k(1)
c     &           ,tau_h2o_cont_1k(1)
c     Debug
            if (cner) then
c     compute NER matrix when required
               call analytic_NER(ai,mu,ref,Nlay,epsilon_ground,
     &              Bg,Bs,B,
     &              tau,
     &              NER)
            else
c     compute only i-space NERs
               call analytic_NER_is(ai,mu,ref,Nlay,epsilon_ground,
     &              Bg,Bs,B,
     &              tau,
     &              NER)
            endif
            if (cflux) then
c     compute upward, downward and net fluxes when required
               call analytic_fluxes(ai,mu,Nlay,pres,tau,B,Bg,Bs,
     &              epsilon_ground,ref,
     &              probe_pressure,probe_layer,probe_altitude,
     &              flux_up,flux_dn,flux_net,
     &              pflux_up,pflux_dn,pflux_net)
c     Debug
c               if (debug) then
c                  write(*,*) 'nu=',nu_ref(ik)
c                  do i=0,Nlev
c                     write(*,*) i,flux_up(i),flux_dn(i),flux_net(i)
c                  enddo ! i
c                  stop
c               endif
c     Debug
            else
c     even when flux computation is not required, compute TOA upward flux only
               call Fasc(ai,mu,Nlay,tau,B,Bg,Bs,epsilon_ground,ref,
     &              Nlay,tau(Nlay),flux_up(Nlay)) ! W/m^2/cm^-1
               call Fdes(ai,mu,Nlay,tau,B,Bg,Bs,epsilon_ground,ref,
     &              Nlay,0.0D+0,flux_dn(Nlay)) ! W/m^2/cm^-1
               flux_net(Nlay)=flux_up(Nlay)-flux_dn(Nlay) ! W/m^2/cm^-1
            endif
c     sending process index 'pindex' to master process
            call MPI_SEND(pindex,1,MPI_INTEGER,0,1,MPI_COMM_WORLD,ierr)
c     sending integer '1' to process 0, meaning partial results will be send now
            call MPI_SEND(one,1,MPI_INTEGER,0,1,MPI_COMM_WORLD,ierr)
c     sendind partial results to the process 0
            call send_partial_results(NER,flux_up,flux_dn,flux_net,
     &           pflux_up,pflux_dn,pflux_net)
c
         else if (codep.eq.3) then ! perform a Rayleigh optical thickness computation
            call MPI_RECV(lay,1,MPI_INTEGER
     &           ,0,MPI_ANY_TAG,MPI_COMM_WORLD,stat,ierr)
            call MPI_RECV(lambda,1,MPI_DOUBLE_PRECISION
     &           ,0,MPI_ANY_TAG,MPI_COMM_WORLD,stat,ierr)
            call calc_tau_rayleigh(pure_co2,gravity,lambda,
     &           pres(lay),pres(lay+1),tau_tmp)
c     sending process index 'pindex' to master process
            call MPI_SEND(pindex,1,MPI_INTEGER,0,1,MPI_COMM_WORLD,ierr)
c     sending integer '1' to process 0, meaning partial results will be send now
            call MPI_SEND(one,1,MPI_INTEGER,0,1,MPI_COMM_WORLD,ierr)
c     sending back result
            call MPI_SEND(tau_tmp,1,MPI_DOUBLE_PRECISION,
     &           0,1,MPI_COMM_WORLD,ierr)
c
         else if (codep.eq.4) then ! perform a CO2 c.i.a. optical thickness computation
            call MPI_RECV(lay,1,MPI_INTEGER
     &           ,0,MPI_ANY_TAG,MPI_COMM_WORLD,stat,ierr)
            call MPI_RECV(wavenumber,1,MPI_DOUBLE_PRECISION
     &           ,0,MPI_ANY_TAG,MPI_COMM_WORLD,stat,ierr)
            call CO2_continuum_optical_thickness(
     &           alt(lay),alt(lay+1),
     &           pres(lay)*x_co2(lay),
     &           pres(lay+1)*x_co2(lay+1),
     &           temp(lay),temp(lay+1),wavenumber,
     &           tau_tmp)
c     sending process index 'pindex' to master process
            call MPI_SEND(pindex,1,MPI_INTEGER,0,1,MPI_COMM_WORLD,ierr)
c     sending integer '1' to process 0, meaning partial results will be send now
            call MPI_SEND(one,1,MPI_INTEGER,0,1,MPI_COMM_WORLD,ierr)
c     sending back result
            call MPI_SEND(tau_tmp,1,MPI_DOUBLE_PRECISION,
     &           0,1,MPI_COMM_WORLD,ierr)
c
         else if (codep.eq.5) then
            call MPI_RECV(lay,1,MPI_INTEGER
     &           ,0,MPI_ANY_TAG,MPI_COMM_WORLD,stat,ierr)
            call MPI_RECV(wavenumber,1,MPI_DOUBLE_PRECISION
     &           ,0,MPI_ANY_TAG,MPI_COMM_WORLD,stat,ierr)
            call H2O_continuum_optical_thickness(
     &           alt(lay),alt(lay+1),
     &           pres(lay),pres(lay+1),
     &           temp(lay),temp(lay+1),wavenumber,
     &           Nmol,mol_name,Nlev,alt,x,
     &           tau_tmp)
c     sending process index 'pindex' to master process
            call MPI_SEND(pindex,1,MPI_INTEGER,0,1,MPI_COMM_WORLD,ierr)
c     sending integer '1' to process 0, meaning partial results will be send now
            call MPI_SEND(one,1,MPI_INTEGER,0,1,MPI_COMM_WORLD,ierr)
c     sending back result
            call MPI_SEND(tau_tmp,1,MPI_DOUBLE_PRECISION,
     &           0,1,MPI_COMM_WORLD,ierr)
c
         else if (codep.eq.6) then
            call MPI_RECV(lay,1,MPI_INTEGER
     &           ,0,MPI_ANY_TAG,MPI_COMM_WORLD,stat,ierr)
            call MPI_RECV(wavenumber,1,MPI_DOUBLE_PRECISION
     &           ,0,MPI_ANY_TAG,MPI_COMM_WORLD,stat,ierr)
            call dimer_CIA_optical_thickness(
     &           N2N2_ndt,N2N2_nu_min,N2N2_nu_max,N2N2_np,
     &           N2N2_T,N2N2_cia,
     &           alt(lay),alt(lay+1),
     &           pres(lay),pres(lay+1),
     &           temp(lay),temp(lay+1),wavenumber,
     &           tau_tmp)
c     sending process index 'pindex' to master process
            call MPI_SEND(pindex,1,MPI_INTEGER,0,1,MPI_COMM_WORLD,ierr)
c     sending integer '1' to process 0, meaning partial results will be send now
            call MPI_SEND(one,1,MPI_INTEGER,0,1,MPI_COMM_WORLD,ierr)
c     sending back result
            call MPI_SEND(tau_tmp,1,MPI_DOUBLE_PRECISION,
     &           0,1,MPI_COMM_WORLD,ierr)
c
         else if (codep.eq.7) then
            call MPI_RECV(lay,1,MPI_INTEGER
     &           ,0,MPI_ANY_TAG,MPI_COMM_WORLD,stat,ierr)
            call MPI_RECV(wavenumber,1,MPI_DOUBLE_PRECISION
     &           ,0,MPI_ANY_TAG,MPI_COMM_WORLD,stat,ierr)
            call dimer_CIA_optical_thickness(
     &           O2O2_ndt,O2O2_nu_min,O2O2_nu_max,O2O2_np,
     &           O2O2_T,O2O2_cia,
     &           alt(lay),alt(lay+1),
     &           pres(lay),pres(lay+1),
     &           temp(lay),temp(lay+1),wavenumber,
     &           tau_tmp)
c     sending process index 'pindex' to master process
            call MPI_SEND(pindex,1,MPI_INTEGER,0,1,MPI_COMM_WORLD,ierr)
c     sending integer '1' to process 0, meaning partial results will be send now
            call MPI_SEND(one,1,MPI_INTEGER,0,1,MPI_COMM_WORLD,ierr)
c     sending back result
            call MPI_SEND(tau_tmp,1,MPI_DOUBLE_PRECISION,
     &           0,1,MPI_COMM_WORLD,ierr)
c
         else ! unknown codep
            write(*,*) 'Error from ',
     &           label(1:strlen(label)),' :'
            write(*,*) 'Unknown codep=',codep
            write(*,*) 'received by process:',pindex
            stop
         endif                  ! codep

c     wait for a new execution code
         goto 333
 666     continue

c     MPICH
      endif ! pindex
c     MPICH

c     MPICH
      call MPI_FINALIZE(ierr)
c     MPICH

      end
