c     Copyright (C)
c     2014-2018 Centre National de la Recherche Scientifique
c     2014-2018 Mesotar
c     
c     This file must be used under the terms of the CeCILL license.
c     This source file is licensed as described in the file COPYING, which
c     you should have received as part of this distribution.  The terms
c     are also available at
c     http://www.cecill.info/licences/Licence_CeCILL_V2.1-en.txt
c
      subroutine get_Cp(Nlev,alt,pres,capa)
      implicit none
      include 'max.inc'
      include 'constants.inc'
c     
c     Purpose: to compute the array of average volumic calorific capacity
c     for each atmospheric layer
c     
c     Input:
c       + Nlev: number of atmospheric levels
c       + alt: altitude at each level (m)
c       + pres: pressure at each level (atm)
c     
c     Output:
c       + capa: average calorific capacity of each layer [J/K/m^3/(time_period)]
c     
c     I/O
      integer Nlev
      double precision alt(1:Nlev_mx)
      double precision pres(1:Nlev_mx)
      double precision capa(1:Nlay_mx)
c     temp
      integer i
      integer nsj
      double precision time_period
      double precision z1,z2
      double precision p1,p2
c     label
      integer strlen
      character*(Nchar_mx) label
      label='subroutine get_Cp'
c     
      nsj=24*3600               ! number of seconds per terrestrial day
      time_period=dble(nsj)     ! final time length for cooling rate [K/time_period]
      do i=1,Nlev-1
         z1=alt(i)
         z2=alt(i+1)
         p1=pres(i)*1.0130D+5 ! atm -> Pa
         p2=pres(i+1)*1.0130D+5   ! atm -> Pa
         capa(i)=Cp*(p1-p2)/(gravity*(z2-z1))/time_period ! J/m^-3/K/[time_period]
      enddo                     ! i
c
c     Debug
      open(10,file='./results/calorific_capacity.txt')
      write(10,*) 'layer / z1 / z2 / Cp [J/m^-3/K/day]'
      do i=1,Nlev-1
         write(10,*) i,alt(i),alt(i+1),capa(i)
      enddo ! i
      close(10)
c     Debug
c     
      return
      end
      
