c     Copyright (C)
c     2014-2018 Centre National de la Recherche Scientifique
c     2014-2018 Mesotar
c     
c     This file must be used under the terms of the CeCILL license.
c     This source file is licensed as described in the file COPYING, which
c     you should have received as part of this distribution.  The terms
c     are also available at
c     http://www.cecill.info/licences/Licence_CeCILL_V2.1-en.txt
c
      subroutine contribution_function(Nlay,NER,alt,cf,omega)
      implicit none
      include 'max.inc'
c
c     Purpose: to compute the contribution function at a given wavenumber.
c     The contribution function is defined as the fraction of energy that leaves the
c     atmosphere as a function of its emission altitude.
c     
c     Input:
c       + Nlay: number of atmospheric layers
c       + NER: NER matrix (W/m^2¨cm^-1)
c       + alt: altitude at each level (m)
c
c     Output:
c       + cf: contribution function, as a function of the emission layer
c       + omega: fraction of the gas-space NER due to each gas layer
c
c     I/O
      integer Nlay
      double precision NER(0:Nlay_mx+1,0:Nlay_mx+1)
      double precision cf(0:Nlay_mx)
      double precision alt(1:Nlev_mx)
      double precision omega(1:Nlay_mx)
c     temp
      integer i
      double precision net_flux(0:Nlay_mx)
      double precision gas_space_ner
c     label
      integer strlen
      character*(Nchar_mx) label
      label='subroutine contribution_function'

c     Compute the net flux @ TOA
      call analytic_net_fluxes(Nlay,NER,net_flux)

c     Compute contribution function
      do i=0,Nlay
c         if ((NER(i,Nlay+1).lt.0.0D+0).and.
c     &        (dabs(NER(i,Nlay+1)).gt.1.0D-6)) then
c            call warning(label)
c            write(*,*) 'NER(',i,',',Nlay+1,')=',NER(i,Nlay+1)
c            write(*,*) 'is not supposed to be negative'
c         endif
c     The contribution function is the Net Exchange between each layer (ground included)
c     and space, normalized by the total (the net flux @ TOA).
         cf(i)=NER(i,Nlay+1)/net_flux(Nlay)
      enddo                     ! i

c     Compute function omega: fraction of the flux emitted by the atmosphere toward space
c     1- get the net power between the atmopshere and space
      gas_space_ner=0.0D+0
      do i=1,Nlay
         gas_space_ner=gas_space_ner+NER(i,Nlay+1)
      enddo ! i
c     2- compute function omega
      do i=1,Nlay
         omega(i)=NER(i,Nlay+1)/gas_space_ner
      enddo ! i
      
      return
      end
