c     Copyright (C)
c     2014-2018 Centre National de la Recherche Scientifique
c     2014-2018 Mesotar
c     
c     This file must be used under the terms of the CeCILL license.
c     This source file is licensed as described in the file COPYING, which
c     you should have received as part of this distribution.  The terms
c     are also available at
c     http://www.cecill.info/licences/Licence_CeCILL_V2.1-en.txt
c
      subroutine read_cia_table(infile,ndt,nu_min,nu_max,np,T,cia)
      implicit none
      include 'max.inc'
c
c     Purpose: to produce a routine capable of reading data from HITRAN2012 CIA tables
c
c     Input:
c       + infile: name of the CIA table to read
c
c     Output:
c       + ndt: number of spectral intervals / temperature points the data is provided for
c       + nu_min: lower wavenumber (inv. cm) for each data interval
c       + nu_max: higher wavenumber (inv. cm) for each data interval
c       + np: number of wavenumber points for each data interval
c       + T: temperature (K) for each data interval
c       + cia: CIA data for each data interval, for each wavenumber point
c         - column 1: wavenumber (inv. cm)
c         - column 2: CIA cross-section (cm^5.molecule^-2)
c     
c     I/O
      character*(Nchar_mx) infile
      double precision nu_min(1:Ndt_int_mx)
      double precision nu_max(1:Ndt_int_mx)
      integer ndt,np(1:Ndt_int_mx)
      double precision T(1:Ndt_int_mx)
      double precision cia(1:Ndt_int_mx,1:Np_mx,1:2)
c     temp
      integer ios
      integer i,j
      character*(Nchar_mx) nlines_file
      logical eof
      integer nlines,nlines_read
      character*(25) s1
c     label
      integer strlen
      character*(Nchar_mx) label
      label='subroutine read_cia_table'
      
      nlines_file='./nlines'
      call get_nlines(infile,nlines_file,nlines)
      open(11,file=infile(1:strlen(infile))
     &     ,status='old',iostat=ios)
      if (ios.ne.0) then        ! file not found
         call error(label)
         write(*,*) 'File could not be found:'
         write(*,*) infile(1:strlen(infile))
         stop
      endif
      nlines_read=0
      ndt=0
      eof=.false.

      do while (.not.eof)
         ndt=ndt+1
         if (ndt.gt.Ndt_int_mx) then
            call error(label)
            write(*,*) 'ndt=',ndt
            write(*,*) 'while Ndt_int_mx=',Ndt_int_mx
            stop
         endif
         read(11,*) s1,nu_min(ndt),nu_max(ndt),np(ndt),T(ndt)
c     Debug
c         write(*,*) nu_min(ndt),nu_max(ndt),np(ndt),T(ndt)
c     Debug
         nlines_read=nlines_read+1
         do i=1,np(ndt)
            read(11,*) (cia(ndt,i,j),j=1,2)
            nlines_read=nlines_read+1
         enddo ! i
         if (nlines_read.ge.nlines) then
            eof=.true.
         endif
      enddo ! while (.not.eof)
      close(11)

      return
      end
