c     Copyright (C)
c     2014-2018 Centre National de la Recherche Scientifique
c     2014-2018 Mesotar
c     
c     This file must be used under the terms of the CeCILL license.
c     This source file is licensed as described in the file COPYING, which
c     you should have received as part of this distribution.  The terms
c     are also available at
c     http://www.cecill.info/licences/Licence_CeCILL_V2.1-en.txt
c
      subroutine read_composition(hs_base,
     &     Nlev,alt,pres,temp)
      implicit none
      include 'max.inc'
      include 'formats.inc'
c
c     Purpose: to read the atmospheric pressure and temperature profiles
c
c     Input:
c       + hs_base: path to the folder that contains atmospheric composition file
c
c     Output:
c       + Nlev: number of atmospheric levels
c       + alt: altitude at each level (m)
c       + pres: pressure at each level (atm)
c       + temp: temperature at each level (K)
c

c     I/O
      character*(Nchar_mx) hs_base
      integer Nlev
      double precision alt(1:Nlev_mx)
      double precision pres(1:Nlev_mx)
      double precision temp(1:Nlev_mx)
c     temp
      integer i,lev,ios,Nmol
      character*(Nchar_mx) file_composition
c     label
      integer strlen
      character*(Nchar_mx) label
      label='subroutine read_composition'

      call add_slash_if_absent(hs_base)
      file_composition=hs_base(1:strlen(hs_base))
     &     //'composition.in'
      open(13,file=file_composition(1:strlen(file_composition)),
     &        status='old',iostat=ios)
      if (ios.ne.0) then        ! file not found
         call error(label)
         write(*,*) 'Data file could not be found:'
         write(*,*) file_composition(1:strlen(file_composition))
         stop
      else
         read(13,*)
         read(13,*)
         read(13,*) Nlev
         read(13,*)
         read(13,*) Nmol
         read(13,*)
         read(13,*)
c     test
         if (Nlev.gt.Nlev_mx) then
            call error(label)
            write(*,*) 'Nlev=',Nlev
            write(*,*) 'while Nlev_mx=',Nlev_mx
            stop
         endif
c     test
         do lev=1,Nlev
            read(13,50) alt(lev) ! km
     &           ,pres(lev) ! atm
     &           ,temp(lev) ! K
            alt(lev)=alt(lev)*1.0D+3 ! km -> m
c            pres(lev)=pres(lev)/1013.0D+0 ! hPa -> atm
         enddo ! lev
      endif
      close(13)

c     analysing pressure
      do lev=2,Nlev
         if (pres(lev).ge.pres(lev-1)) then
            call warning(label)
            write(*,*) 'pressure profile is incoherent'
            goto 111
         endif
      enddo ! lev
 111  continue
c     analysing altitudes
      do lev=2,Nlev
         if (alt(lev).le.alt(lev-1)) then
            call warning(label)
            write(*,*) 'altitude profile is incoherent'
            goto 222
         endif
      enddo ! lev
 222  continue

      return
      end
