c     Copyright (C)
c     2014-2018 Centre National de la Recherche Scientifique
c     2014-2018 Mesotar
c     
c     This file must be used under the terms of the CeCILL license.
c     This source file is licensed as described in the file COPYING, which
c     you should have received as part of this distribution.  The terms
c     are also available at
c     http://www.cecill.info/licences/Licence_CeCILL_V2.1-en.txt
c
      subroutine read_data(nu_min,nu_max,Tg,Ts,em_g,abs_cont,
     &     ks_coeff,probe_pressure,mu)
      implicit none
      include 'max.inc'
      include 'formats.inc'
c
c     Purpose: to read input data
c
c     Input:
c
c     Output:
c       + nu_min,nu_max: limits of the spectral interval (inverse centimeters)
c         radiative transfer computation must be held for.
c       + Tg: ground temperature [K]
c       + Ts: space temperature [K]
c       + em_g: ground emissivity
c       + abs_cont: absorption continuum (cm^-1/amg^2)
c       + ks_coeff: division factor for scattering optical thicknesses
c       + probe_pressure: value of the probe pressure (atm)
c       + mu: cos(theta) if angular integration has been disabled (0-1)
c
c     I/O
      double precision nu_min
      double precision nu_max
      double precision Tg,Ts
      double precision em_g
      double precision abs_cont
      double precision ks_coeff
      double precision probe_pressure
      double precision mu
c     temp
      character*(Nchar_mx) dt_file
      integer ios,i
c     label
      integer strlen
      character*(Nchar_mx) label
      label='subroutine read_data'

      dt_file='./data.in'
      open(10,file=dt_file(1:strlen(dt_file))
     &     ,status='old',iostat=ios)
      if (ios.ne.0) then        ! file not found
         call error(label)
         write(*,*) 'File not found:'
         write(*,*) dt_file(1:strlen(dt_file))
         stop
      else
         do i=1,3
            read(10,*)
         enddo
         read(10,*) nu_min
         read(10,*)
         read(10,*) nu_max
         read(10,*)
         read(10,*) abs_cont
         read(10,*)
         read(10,*) ks_coeff
         read(10,*)
         read(10,*) probe_pressure
         read(10,*)
         read(10,*) mu
      endif
      close(10)
      
      dt_file='./data/hires_spectra/profile_parameters.in'
      open(11,file=dt_file(1:strlen(dt_file))
     &     ,status='old',iostat=ios)
      if (ios.ne.0) then        ! file not found
         call error(label)
         write(*,*) 'File not found:'
         write(*,*) dt_file(1:strlen(dt_file))
         stop
      else
         do i=1,3
            read(11,*)
         enddo
         read(11,*) Tg
         read(11,*)
         read(11,*) Ts
         read(11,*)
         read(11,*) em_g
      endif
      close(11)
         
      if (nu_max.le.nu_min) then
         call error(label)
         write(*,*) 'Bad input parameters:'
         write(*,*) 'nu_min=',nu_min
         write(*,*) 'nu_max=',nu_max
         stop
      endif
      if (nu_min.le.0.0D+0) then
         call error(label)
         write(*,*) 'Bad input parameter:'
         write(*,*) 'nu_min=',nu_min
         stop
      endif
      if (Tg.le.0.0D+0) then
         call error(label)
         write(*,*) 'Bad input parameter:'
         write(*,*) 'Tg=',Tg
         stop
      endif
      if (Ts.lt.0.0D+0) then
         call error(label)
         write(*,*) 'Bad input parameter:'
         write(*,*) 'Ts=',Ts
         stop
      endif
      if (Tg.le.Ts) then
         call error(label)
         write(*,*) 'Inconsistent input parameter:'
         write(*,*) 'Tg=',Tg,' < Ts=',Ts
         stop
      endif
      if ((em_g.lt.0.0D+0).or.(em_g.gt.1.0D+0)) then
         call error(label)
         write(*,*) 'Bad input parameter:'
         write(*,*) 'em_g=',em_g
         stop
      endif
      if (abs_cont.lt.0.0D+0) then
         call error(label)
         write(*,*) 'Bad input parameter:'
         write(*,*) 'abs_cont=',abs_cont
         stop
      endif
      if (ks_coeff.lt.0.0D+0) then
         call error(label)
         write(*,*) 'Bad input parameter:'
         write(*,*) 'ks_coeff=',ks_coeff
         stop
      endif
      if (probe_pressure.lt.0.0D+0) then
         call error(label)
         write(*,*) 'probe_pressure=',probe_pressure
         write(*,*) '< 0'
         stop
      endif
      if (probe_pressure.gt.1.0D+0) then
         call error(label)
         write(*,*) 'probe_pressure=',probe_pressure
         write(*,*) '> 1 atm'
         stop
      endif
      if ((mu.le.0.0D+0).or.(mu.gt.1.0D+0)) then
         call error(label)
         write(*,*) 'mu=',mu
         write(*,*) 'should be in the ]0-1] range'
         stop
      endif

      return
      end
