c     Copyright (C)
c     2014-2018 Centre National de la Recherche Scientifique
c     2014-2018 Mesotar
c     
c     This file must be used under the terms of the CeCILL license.
c     This source file is licensed as described in the file COPYING, which
c     you should have received as part of this distribution.  The terms
c     are also available at
c     http://www.cecill.info/licences/Licence_CeCILL_V2.1-en.txt
c
      subroutine detect_co2(Nlev,Nmol,mol_name,x,
     &     co2_found,pure_co2,x_co2)
      implicit none
      include 'max.inc'
c
c     Purpose: to detect CO2 in the gas mixture
c
c     Input:
c       + Nlev: number of levels
c       + Nmol: number of molecules radiatively active
c       + mol_name: name of each molecule
c       + x: molar fraction (mixratio) of each molecule, at each atmospheric level
c
c     Output:
c       + co2_found: true if CO2 has been identified, with a positive molar fraction
c       + pure_co2: true if the molar fraction of CO2 is > 0.90
c       + x_co2: CO2 molar fraction @ each level
c
c     I/O
      integer Nlev,NMol
      character*10 mol_name(1:Nmol_mx)
      double precision x(1:Nlev_mx,1:Nmol_mx)
      logical co2_found,pure_co2
      double precision x_co2(1:Nlev_mx)
c     temp
      integer lev,mol
      integer lev0,mol0
c     label
      integer strlen
      character*(Nchar_mx) label
      label='subroutine detect_co2'

      co2_found=.false.
      do mol=1,Nmol
         if (mol_name(mol)(1:strlen(mol_name(mol))).eq.'CO2') then
            do lev=1,Nlev
               if (x(lev,mol).gt.0.0D+0) then
                  co2_found=.true.
                  mol0=mol
                  lev0=lev
                  goto 111
               endif
            enddo ! lev
         endif
      enddo ! mol
 111  continue
      if (co2_found) then
         do lev=1,Nlev
            x_co2(lev)=x(lev,mol0)
         enddo ! lev
      endif ! co2_found
      pure_co2=.false.
      if (co2_found) then
         if (x_co2(lev).ge.0.90D+0) then
            pure_co2=.true.
         endif
      endif

      return
      end



      subroutine detect_h2o(Nlev,Nmol,mol_name,x,
     &     h2o_found,x_h2o)
      implicit none
      include 'max.inc'
c
c     Purpose: to detect H2O in the gas mixture
c
c     Input:
c       + Nlev: number of levels
c       + Nmol: number of molecules radiatively active
c       + mol_name: name of each molecule
c       + x: molar fraction (mixratio) of each molecule, at each atmospheric level
c
c     Output:
c       + h2o_found: true if H2O has been identified, with a positive molar fraction
c       + x_h2o: H2O molar fraction @ each level
c
c     I/O
      integer Nlev,NMol
      character*10 mol_name(1:Nmol_mx)
      double precision x(1:Nlev_mx,1:Nmol_mx)
      logical h2o_found
      double precision x_h2o(1:Nlev_mx)
c     temp
      integer lev,mol,mol0
c     label
      integer strlen
      character*(Nchar_mx) label
      label='subroutine detect_h2o'

      h2o_found=.false.
      do mol=1,Nmol
         if (mol_name(mol)(1:strlen(mol_name(mol))).eq.'H2O') then
            do lev=1,Nlev
               if (x(lev,mol).gt.0.0D+0) then
                  h2o_found=.true.
                  mol0=mol
                  goto 111
               endif
            enddo ! lev
         endif
      enddo ! mol
 111  continue
      if (h2o_found) then
         do lev=1,Nlev
            x_h2o(lev)=x(lev,mol0)
         enddo ! lev
      endif ! h2o_found

      return
      end
